package be.authtest.vaadin.oauth;

import java.io.IOException;
import java.security.Principal;
import java.security.acl.Group;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.TextInputCallback;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.login.FailedLoginException;
import javax.security.auth.login.LoginException;
import javax.security.auth.spi.LoginModule;
import javax.sql.DataSource;

import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jboss.security.NestableGroup;
import org.jboss.security.SecurityConstants;
import org.jboss.security.SimpleGroup;
import org.jboss.security.SimplePrincipal;
import org.scribe.builder.ServiceBuilder;
import org.scribe.model.OAuthRequest;
import org.scribe.model.Response;
import org.scribe.model.Token;
import org.scribe.model.Verb;
import org.scribe.model.Verifier;
import org.scribe.oauth.OAuthService;

import com.google.gson.Gson;

/**
 * JDBC login module to authenticate LPOS terminals with JAAS <br/>
 * 
 * @author LDK
 *
 */
public class GoogleLoginModule implements LoginModule {

	private String gpluskey = "********************************";
	private String gplussecret = "*******************************";

	private static final int CODE = 0;
	private static final Logger logger = LogManager.getLogger(GoogleLoginModule.class);
	private boolean authenticationSuccess = false;
	protected Subject subject;
	protected Map<String, ?> sharedState;
	protected Map<String, ?> options;
	protected CallbackHandler callbackHandler;
	OAuthService service;
	
	String username = "";
	ArrayList<String> roles = null;
	

	// Options from standalone-full.xml
	private String dsJndiName = "java:jboss/datasources/frontDS";
	private String rolesQuery = "SELECT * from lbs_user_role";
	private String callbackUrl = "";

	public GoogleLoginModule() {
		logger.debug("Instantiating login module {} {}", getClass().getName(), this);
	}

	@Override
	public void initialize(Subject _subject, CallbackHandler _callbackHandler, Map<String, ?> _sharedState, Map<String, ?> _options) {
		this.subject = _subject;
		this.callbackHandler = _callbackHandler;
		this.sharedState = _sharedState;
		this.options = _options;
		this.dsJndiName = (String)this.options.get("dsJndiName");
		this.rolesQuery = (String) this.options.get("rolesQuery");
		this.callbackUrl = (String) this.options.get("callbackUrl");
		this.service = createService();
	}

	/**
	 * This method is called via the JAAS login <br/>
	 * LoginContext loginContext = new LoginContext(LOGIN_MODULE_MC,
	 * callbackHandler); <br/>
	 * loginContext.login(); <br/>
	 * The NameCallback and PasswordCallback are populated with dummy prompts
	 * (prompts are not used).
	 * 
	 */
	@Override
	public boolean login() throws LoginException {

		Callback[] callbackArray = new Callback[1];
		callbackArray[CODE] =  new TextInputCallback("code");

		try {
			this.callbackHandler.handle(callbackArray);
			String code = ((TextInputCallback) callbackArray[CODE]).getText();
			
			Verifier v = new Verifier(code);
			Token t = getService().getAccessToken(null, v);
			OAuthRequest r = new OAuthRequest(Verb.GET, "https://www.googleapis.com/plus/v1/people/me");
			getService().signRequest(t, r);
			Response resp = r.send();
			GooglePlusAnswer answer = new Gson().fromJson(resp.getBody(), GooglePlusAnswer.class);

			if (!verifyGoogleAnswer(answer)) {
				throw new FailedLoginException("Authorization failure for user {" + answer.emails[0].value + "}");
			}
			
			this.username = answer.emails[0].value;
			this.roles = getUserRoles();
			
			this.authenticationSuccess = true;
			return this.authenticationSuccess;

		} catch (IOException ioex) {
			ioex.printStackTrace();
			
		} catch (SQLException sex) {
			sex.printStackTrace();
		}
		catch (UnsupportedCallbackException unex) {
			unex.printStackTrace();
		}
		return false;
	}

	@Override
	public boolean commit() throws LoginException {

		Set<Principal> principals = this.subject.getPrincipals();
		if (this.authenticationSuccess) {
			// Add principals to subject
			SimplePrincipal caller = new SimplePrincipal(this.username);
			principals.add(caller);

			Group[] roleSets = getRoleSets();

			for (int g = 0; g < roleSets.length; g++) {
				Group group = roleSets[g];
				String name = group.getName();
				Group subjectGroup = createGroup(name, principals);
				if (subjectGroup instanceof NestableGroup) {
					SimpleGroup tmp = new SimpleGroup("Roles");
					subjectGroup.addMember(tmp);
					subjectGroup = tmp;
				}
				// Copy the group members to the Subject group
				Enumeration<? extends Principal> members = group.members();
				while (members.hasMoreElements()) {
					Principal role = (Principal) members.nextElement();
					subjectGroup.addMember(role);
				}
			}
			// add the CallerPrincipal group if none has been added in
			// getRoleSets
			Group callerGroup = getCallerPrincipalGroup(principals);
			if (callerGroup == null) {
				callerGroup = new SimpleGroup(SecurityConstants.CALLER_PRINCIPAL_GROUP);
				callerGroup.addMember(new SimplePrincipal(this.username));
				principals.add(callerGroup);
			}

		}
		return this.authenticationSuccess;
	}

	private Group[] getRoleSets() {
		Group[] groups = {new SimpleGroup("Roles")};
		
		for (String role : this.roles) {
			SimplePrincipal spr = new SimplePrincipal(role);
			groups[0].addMember(spr);
		}
		
		return groups;
	}
	
	@Override
	public boolean abort() throws LoginException {
		return this.authenticationSuccess;
	}

	@Override
	public boolean logout() throws LoginException {
		this.subject.getPrincipals().clear();
		return true;
	}

	public Subject getSubject() {
		return this.subject;
	}

	
	protected ArrayList<String> getUserRoles() throws LoginException, SQLException {
		Connection conn = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		ArrayList<String> _roles = new ArrayList<String>(); 
		try {
			InitialContext ctx = new InitialContext();
			DataSource ds = (DataSource) ctx.lookup(this.dsJndiName);
			conn = ds.getConnection();
			// Get the password
			ps = conn.prepareStatement(this.rolesQuery);
			int i = 1;
			ps.setString(i++, this.username);
			rs = ps.executeQuery();
			
			while (rs.next()) {
				_roles.add(rs.getString(1));
			}
			
			if (_roles.isEmpty()) {
				throw new FailedLoginException("No roles found in Principals");
			}			
			
		} catch (NamingException ex) {
			throw new LoginException(ex.toString(true));
		} catch (SQLException ex) {
			throw new LoginException(ex.toString());
		} finally {
			if (rs != null) {
				rs.close();
			}
			if (ps != null) {
				ps.close();
			}
			if (conn != null) {
				conn.close();
			}
		}
		return _roles;
	}

	private OAuthService getService() {
		return this.service;
	}

	private boolean verifyGoogleAnswer(GooglePlusAnswer answer) {
		if (answer != null && answer.emails.length > 0 && StringUtils.isNotEmpty(answer.emails[0].value)) {
			return true;
		}
		return false;
	}
	
    private OAuthService createService() {
        ServiceBuilder sb = new ServiceBuilder();
        sb.provider(Google2Api.class);
        sb.apiKey(this.gpluskey);
        sb.apiSecret(this.gplussecret);
        sb.scope("email");
        sb.callback(getCallbackUrl());
        return sb.build();
    }

	private String getCallbackUrl() {
		return this.callbackUrl;
	}

	protected Group createGroup(String name, Set<Principal> principals) {
		Group roles = null;
		Iterator<Principal> iter = principals.iterator();
		while (iter.hasNext()) {
			Object next = iter.next();
			if ((next instanceof Group) == false)
				continue;
			Group grp = (Group) next;
			if (grp.getName().equals(name)) {
				roles = grp;
				break;
			}
		}
		// If we did not find a group create one
		if (roles == null) {
			roles = new SimpleGroup(name);
			principals.add(roles);
		}
		return roles;
	}

	protected Group getCallerPrincipalGroup(Set<Principal> principals) {
		Group callerGroup = null;
		for (Principal principal : principals) {
			if (principal instanceof Group) {
				Group group = Group.class.cast(principal);
				if (group.getName().equals(SecurityConstants.CALLER_PRINCIPAL_GROUP)) {
					callerGroup = group;
					break;
				}
			}
		}
		return callerGroup;
	}

}
