package be.authtest.vaadin.ui;

import java.util.List;

import javax.inject.Inject;

import com.vaadin.annotations.Theme;
import com.vaadin.annotations.Widgetset;
import com.vaadin.cdi.CDIUI;
import com.vaadin.server.FontAwesome;
import com.vaadin.server.VaadinRequest;
import com.vaadin.ui.Button;
import com.vaadin.ui.CssLayout;
import com.vaadin.ui.UI;
import com.vaadin.ui.themes.ValoTheme;
import com.vaadin.v7.data.util.BeanItemContainer;
import com.vaadin.v7.ui.Grid;
import com.vaadin.v7.ui.HorizontalLayout;
import com.vaadin.v7.ui.TextField;
import com.vaadin.v7.ui.VerticalLayout;

import be.authtest.vaadin.ejb.AuthService;
import be.authtest.vaadin.entity.LBSPlayer;

@Theme("valo")
@CDIUI("pm")
@SuppressWarnings("serial")
@Widgetset("com.vaadin.v7.Vaadin7WidgetSet")
public class PlayerAdminUI extends UI implements PlayerAdmin {

	@Inject
	private
	AuthService authService;
	
	private Grid grid;
	private PlayerForm playerForm;
	
	@Override
	protected void init(VaadinRequest request) {
		final VerticalLayout layout = new VerticalLayout();
		layout.setMargin(true);
		layout.setSpacing(true);
		this.playerForm = new PlayerForm(this, this.authService);
		createGrid();
		
		HorizontalLayout content = new HorizontalLayout(this.grid, this.playerForm);
		content.setSpacing(true);
		content.setSizeFull();
		this.grid.setSizeFull();
		content.setExpandRatio(this.grid, 1);
		
		TextField filterText = new TextField();
		filterText.setInputPrompt("Filter by personalId...");
		filterText.addTextChangeListener(e -> {
			this.grid.setContainerDataSource(new BeanItemContainer<>(LBSPlayer.class, this.authService.findByPersonalId(e.getText())));
		});
		Button clearFilterTextBtn = new Button(FontAwesome.TIMES);
		clearFilterTextBtn.setDescription("Clear the current filter");
		clearFilterTextBtn.addClickListener(e -> {
		  filterText.clear();
		  updateList();
		});

		Button refresh = new Button("Refresh");
		refresh.addClickListener(e -> {
			updateList();
		});


		CssLayout filtering = new CssLayout();
		filtering.addComponents(filterText, clearFilterTextBtn);
		filtering.setStyleName(ValoTheme.LAYOUT_COMPONENT_GROUP);
		
		layout.addComponents(filtering, content, refresh);
		updateList();		
		setContent(layout);
		this.playerForm.setVisible(false);
		
		this.grid.addSelectionListener(event -> {
			if (event.getSelected().isEmpty()) {
				this.playerForm.setVisible(false);
			}
			else {
				LBSPlayer player = (LBSPlayer)event.getSelected().iterator().next();
				this.playerForm.setPlayer(player);
			}
		});
		
	}

	public AuthService getVaadinService() {
		return this.authService;
	}

	@Override
	public void updateList() {
		List<LBSPlayer> list = this.authService.allRecords();
		this.grid.setContainerDataSource(new BeanItemContainer<>(LBSPlayer.class, list));
	}
	
	private void createGrid() {
		this.grid = new Grid();
		this.grid.setColumns("personalId", "username", "venue", "firstName", "lastName", "birthDate", "hasImsAccount", "status", "created");
		
		/*
		Grid.Column picture = 
		this.grid.addColumn("picture").setRenderer(new ImageRenderer(), new Converter<Resource, String>() {
			@Override
			public String convertToModel(Resource value, Class<? extends String> targetType, Locale l)
					throws Converter.ConversionException {
				return "not needed";
			}

			@Override
			public Resource convertToPresentation(String value, Class<? extends Resource> targetType, Locale l)
					throws Converter.ConversionException {
				return new ExternalResource(value);
			}

			@Override
			public Class<String> getModelType() {
				return String.class;
			}

			@Override
			public Class<Resource> getPresentationType() {
				return Resource.class;
			}
		});
		*/
		
		// this.grid.setFrozenColumnCount(4);
		// Override style
	}

	@Override
	public void unselect() {
		this.grid.select(null);
		
	}

}
